#include "ADF4351.h"

void init_ADF4351()
{
    setPinDir(PORT_CLK, PIN_CLK, GPIO_OUTPUT);
    setPinDir(PORT_DATA, PIN_DATA, GPIO_OUTPUT);
    setPinDir(PORT_LE, PIN_LE, GPIO_OUTPUT);
    setPinDir(PORT_CE, PIN_CE, GPIO_OUTPUT);
    setPinDir(PORT_RDB_RF, PIN_RDB_RF, GPIO_OUTPUT);
    setPinDir(PORT_LD, PIN_LD, GPIO_INPUT);
    
    setPinState(PORT_LE, PIN_LE, 0);
    setPinState(PORT_CLK, PIN_CLK, 0);
    setPinState(PORT_DATA, PIN_DATA, 0);
    
    // enable the chip
    setPinState(PORT_CE, PIN_CE, 1);
    
    // unmute RF output
    setPinState(PORT_RDB_RF, PIN_RDB_RF, 1);
}

void send_word_to_ADF4351(uint32_t word)
{
    // set LE low to enable Shift registers
    setPinState(PORT_LE, PIN_LE, 0);
    wait_cycles(100);
    
    for(uint32_t i = 0; i < 32; i++)
    {
        setPinState(PORT_CLK, PIN_CLK, 0);
        wait_cycles(100);
        
        uint32_t cb = (uint32_t)0x01 & (word >> (31 - i));
        setPinState(PORT_DATA, PIN_DATA, cb);
        wait_cycles(100);
        
        setPinState(PORT_CLK, PIN_CLK, 1);
        wait_cycles(100);
    }
    
    // set LE high to enable Shift registers
    setPinState(PORT_LE, PIN_LE, 1);
    wait_cycles(100);
}

uint8_t get_lock_status_ADF4351()
{
    return getPinState(PORT_LD, PIN_LD);
}

void set_frequency_ADF4351(uint32_t freq_kHz)
{
    /*
        send_word_to_ADF4351(0x580005);
        send_word_to_ADF4351(0xC50034);
        send_word_to_ADF4351(0x0004B3);
        send_word_to_ADF4351(0x004042);
        send_word_to_ADF4351(0x008029);
        send_word_to_ADF4351(0x730010);
    */
    
    uint32_t ref_clk_khz = 10000;    // Ref_clk = 10MHz
    
    uint32_t rf_div_level = 0;
    while((freq_kHz * (1 << rf_div_level)) < 2200000)
    {
        rf_div_level++;
    }
    
    uint32_t channel_spacing_after_divider_kHz;
    if((1 << rf_div_level) >= 4)
    {
        channel_spacing_after_divider_kHz = 1;
    }
    else
    {
        channel_spacing_after_divider_kHz = 2;
    }
    uint32_t channel_spacing_before_divider_kHz = channel_spacing_after_divider_kHz * (ref_clk_khz / (1 << rf_div_level));    // Channel Spacing (before divider!)
    
    uint32_t INT_VALUE = (1 << rf_div_level) * freq_kHz / ref_clk_khz;
    uint32_t mod = ref_clk_khz / channel_spacing_before_divider_kHz;
    uint32_t FRAC_VALUE = ((((1 << rf_div_level) * (freq_kHz * 10000) / ref_clk_khz) - (10000 * INT_VALUE)) * mod) / 10000;
    
    uint32_t PHASE_ADJUST = 0x00;    // OFF
    uint32_t PRESCALER_BIT = 0x00;    // 4/5
    uint32_t PHASE_VALUE = 0x01;    // 1 (Recommended)
    uint32_t MOD_VALUE = mod;
    
    uint32_t LOW_NOISE_AND_LOW_SPUR_MODES = 0x00;    // Low Noise Mode
    uint32_t MUXOUT = 0x00;    // Three-State Output
    uint32_t REFERENCE_DOUBLE_BIT = 0x00;    // Disabled
    uint32_t REF_DIV_BIT = 0x00;    // Disabled
    uint32_t R_CNT_VALUE = 0x01;    // 1
    uint32_t DOUBLE_BUFFER_BIT = 0x00;    // Disabled
    uint32_t CHARGE_PUMP_CURRENT_SETTING = 0x07;    // 2.5mA
    uint32_t LDF_BIT = 0x00;    // FRAC-N
    uint32_t LDP_BIT = 0x00;    // 10ns
    uint32_t PD_POLARITY_BIT = 0x01;    // Positive
    uint32_t POWER_DOWN_BIT = 0x00;    // Disabled
    uint32_t CP_THREE_STATE_BIT = 0x00;    // Disabled
    uint32_t COUNTER_RESET_BIT = 0x00;    // Disabled
    
    uint32_t BAND_SELECT_CLOCK_MODE_BIT = 0x00;    // Low
    uint32_t ABP_BIT = 0x00;    // 6ns (FRAC-N)
    uint32_t CHARGE_CANCEL_BIT = 0x00;    // Disabled
    uint32_t CSR_BIT = 0x00;    // Disabled
    uint32_t CLK_DIV_MODE = 0x00;    // Clock Divider Off
    uint32_t CLOCK_DIVIDER_VALUE = 150;    // 150
    
    uint32_t FEEDBACK_SELECT_BIT = 0x01;    // Fundamental
    uint32_t RF_DIVIDER_SELECT = rf_div_level;
    uint32_t BAND_SELECT_CLOCK_DIVIDER_VALUE = ref_clk_khz / 125;    // It seems to be required to have 125kHz Band Selector Clock;
    uint32_t VCO_POWER_DOWN_BIT = 0x00;    // VCO Powered Up
    uint32_t MTLD_BIT = 0x00;    // Mute Till Lock Detect = Disabled
    uint32_t AUX_OUTPUT_SELECT_BIT = 0x00;    // Divided Output
    uint32_t AUX_OUTPUT_ENABLE_BIT = 0x00;    // Disabled
    uint32_t AUX_OUTPUT_POWER = 0x00;    // -4dBm
    uint32_t RF_OUTPUT_ENABLE_BIT = 0x01;    // Enabled
    uint32_t OUTOUT_POWER = 0x02;    // +2dBm
    
    uint32_t LD_PIN_MODE = 0x01;    // Digital Lock Detect
    
    
    
    uint32_t reg_0 = ((0xFFFF & INT_VALUE) << 15) | ((0xFFF & FRAC_VALUE) << 3) | 0x0;
    
    uint32_t reg_1 = ((0x01 & PHASE_ADJUST) << 28) | ((0x01 & PRESCALER_BIT) << 27)
                   | ((0xFFF & PHASE_VALUE) << 15) | ((0xFFF & MOD_VALUE) << 3) | 0x1;
    
    uint32_t reg_2 = ((0x03 & LOW_NOISE_AND_LOW_SPUR_MODES) << 29) | ((0x07 & MUXOUT) << 26)
                   | ((0x01 & REFERENCE_DOUBLE_BIT) << 25) | ((0x01 & REF_DIV_BIT) << 24)
                   | ((0x3FF & R_CNT_VALUE) << 14) | ((0x01 & DOUBLE_BUFFER_BIT) << 13)
                   | ((0x0F & CHARGE_PUMP_CURRENT_SETTING) << 9) | ((0x01 & LDF_BIT) << 8)
                   | ((0x01 & LDP_BIT) << 7) | ((0x01 & PD_POLARITY_BIT) << 6)
                   | ((0x01 & POWER_DOWN_BIT) << 5) | ((0x01 & CP_THREE_STATE_BIT) << 4)
                   | ((0x01 & COUNTER_RESET_BIT) << 3) | 0x2;
    
    uint32_t reg_3 = ((0x01 & BAND_SELECT_CLOCK_MODE_BIT) << 23) | ((0x01 & ABP_BIT) << 22)
                   | ((0x01 & CHARGE_CANCEL_BIT) << 21) | ((0x01 & CSR_BIT) << 18)
                   | ((0x03 & CLK_DIV_MODE) << 15) | ((0xFFF & CLOCK_DIVIDER_VALUE) << 3) | 0x3;
    
    uint32_t reg_4 = ((0x01 & FEEDBACK_SELECT_BIT) << 23) | ((0x07 & RF_DIVIDER_SELECT) << 20)
                   | ((0xFF & BAND_SELECT_CLOCK_DIVIDER_VALUE) << 12) | ((0x01 & VCO_POWER_DOWN_BIT) << 11)
                   | ((0x01 & MTLD_BIT) << 10) | ((0x01 & AUX_OUTPUT_SELECT_BIT) << 9)
                   | ((0x01 & AUX_OUTPUT_ENABLE_BIT) << 8) | ((0x03 & AUX_OUTPUT_POWER) << 6)
                   | ((0x01 & RF_OUTPUT_ENABLE_BIT) << 5) | ((0x03 & OUTOUT_POWER) << 3) | 0x4;
    
    uint32_t reg_5 = ((0x03 & LD_PIN_MODE) << 22) | 0x5;
    
    
    
    send_word_to_ADF4351(reg_5);
    send_word_to_ADF4351(reg_4);
    send_word_to_ADF4351(reg_3);
    send_word_to_ADF4351(reg_2);
    send_word_to_ADF4351(reg_1);
    send_word_to_ADF4351(reg_0);
}
