#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "STM32F100.h"
#include "LCD.h"
#include "ADF4351.h"
#include "AD9951.h"

#define ROTATION_NONE   0
#define ROTATION_RIGHT  1
#define ROTATION_LEFT   2
#define SW_DOWN         3

#define RE_PORT_A       PORTC
#define RE_PIN_A        2
#define RE_PORT_B       PORTC
#define RE_PIN_B        3
#define RE_PORT_SW      PORTA
#define RE_PIN_SW       3

void init_rotation_encoder()
{
    setPinDir(RE_PORT_A, RE_PIN_A, GPIO_INPUT);
    setPinDir(RE_PORT_B, RE_PIN_B, GPIO_INPUT);
    setPinDir(RE_PORT_SW, RE_PIN_SW, GPIO_INPUT);
}

uint8_t wait_for_event()
{
    uint8_t a, b, sw;
    uint32_t timeout = 0;
    do
    {
        a = getPinState(RE_PORT_A, RE_PIN_A);
        b = getPinState(RE_PORT_B, RE_PIN_B);
        sw = getPinState(RE_PORT_SW, RE_PIN_SW);
        timeout++;
    } while((a == 1) && (b == 1) && (sw == 1) && (timeout < 200000));
    
    uint8_t result = ROTATION_NONE;
    if((a == 1) && (b == 0))
    {
        while(getPinState(RE_PORT_B, RE_PIN_B) == 0);
        result = ROTATION_RIGHT;
    }
    else if((a == 0) && (b == 1))
    {
        while(getPinState(RE_PORT_A, RE_PIN_A) == 0);
        result = ROTATION_LEFT;
    }
    else if(sw == 0)
    {
        result = SW_DOWN;
    }
    
    return result;
}

int main()
{
    setup_clock_HSI(3, 1);
    
    initGPIO();
    init_rotation_encoder();
    
    initLCD();
    write_string_LCD("   Frequency       Generator    ");
    
    init_ADF4351();
    init_AD9951();
    
    uint32_t freq = 14000;    // start frequency in kHz
    //set_frequency_ADF4351(freq);
    set_frequency_ADF4351(149200);
    configure_AD9951(freq);
    uint8_t event;
    uint8_t sw_state;
    uint32_t freq_step = 10000;
    
    while(1)
    {
        sw_state = getPinState(RE_PORT_SW, RE_PIN_SW);
        
        event = wait_for_event();
        switch(event)
        {
            case ROTATION_RIGHT:
                freq += freq_step;
                //set_frequency_ADF4351(freq);
                configure_AD9951(freq);
            break;
            
            case ROTATION_LEFT:
                freq -= freq_step;
                //set_frequency_ADF4351(freq);
                configure_AD9951(freq);
            break;
            
            case SW_DOWN:
                if(sw_state == 1)
                {
                    switch(freq_step)
                    {
                        case 10000:
                            freq_step = 1000;
                        break;
                        case 1000:
                            freq_step = 100;
                        break;
                        case 100:
                            freq_step = 10;
                        break;
                        case 10:
                            freq_step = 1;
                        break;
                        case 1:
                            freq_step = 10000;
                        break;
                    }
                }
            break;
        }
        
        write_freq_locked_LCD(freq, get_lock_status_ADF4351(), freq_step);
    }
    
    return 0;
}
